import sys

try:
    import pygtk
    #tell pyGTK, if possible, that we want GTKv2
    pygtk.require("2.0")
except:
    #Some distributions come with GTK2, but not pyGTK
    pass

try:
    import gtk
    import gtk.glade
    import gobject
except:
    print "An error occured"
    sys.exit(1)

import gnome.ui
from misc import GuyumConst
import guyumquery
import guyumpackage
from gyfunctions import *


'''
#
# dependencies treemodel
'''
class DepsTree:
    _DEP_NAME=0
    _DEP_REMOTEVERSION=1
    _DEP_SIZE=2
    _DEP_PKGID=3
        
    '''
    #
    # constructor 
    ''' 
    def __init__(self, wTree):
        self.model = gtk.TreeStore( gobject.TYPE_STRING,
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING,
                                    gobject.TYPE_STRING)
        self.wTree = wTree
            
    '''
    #
    # add columns to treeview 
    ''' 
    def add_columns( self ):
        self.treeview = gtk.TreeView()
        self.treeview.set_rules_hint(True)
                
        # name column
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn( 'Name', renderer, text=self._DEP_NAME )
        column.set_clickable(True)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column.set_spacing(2)        
        column.connect("clicked", treeviewSortIDSet, (self.treeview, self._DEP_NAME))
        self.treeview.append_column(column)
        
        # remoteversion column
        column = gtk.TreeViewColumn( 'Remote', renderer, text=self._DEP_REMOTEVERSION )
        column.set_clickable(True)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column.set_spacing(2)        
        self.treeview.append_column(column)

        # download size column
        column = gtk.TreeViewColumn( 'Download size', renderer, text=self._DEP_SIZE )
        column.set_clickable(True)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column.set_spacing(2)        
        self.treeview.append_column(column)

        # pkgid column
        column = gtk.TreeViewColumn( '', renderer, text=self._DEP_PKGID )
        column.set_visible(False)        
        self.treeview.append_column(column)
        
       

    '''
    #
    # add data to treemodel
    '''
    def add_data(self, data):
        iter = None
        for package in data:
            iter = self.model.append(None)                      
            self.model.set(iter,
                           self._DEP_NAME, package.name,
                           self._DEP_REMOTEVERSION, package.dispremoteversion,
                           self._DEP_SIZE, package.compsize,
                           self._DEP_PKGID, package.packageid )    
        sortmodel = gtk.TreeModelSort(self.model)
        self.treeview.set_model(sortmodel)
        sortmodel.set_sort_column_id(self._DEP_NAME, gtk.SORT_ASCENDING)        
        
    '''
    #
    # display package data
    '''
    def _set_package_info(self, repoi, packageid):
        self.tooltips = gtk.Tooltips()
        package = repoi.getnamedpackage(packageid)

        writeListIntoTree(self.wTree.get_widget("deps_treeview_conflicts"), package.conflicts)
        writeIntoTextBuffer(self.wTree.get_widget("deps_textview_description"), package.description)
        
        if package.group!=None and package.group!="(none)":
            (self.wTree.get_widget("deps_entry_category")).set_text(package.group)
        else:
            (self.wTree.get_widget("deps_entry_category")).set_text("")
        
        if package.licenses!=None and package.licenses!="(none)":
            (self.wTree.get_widget("deps_entry_licenses")).set_text(package.licenses)
        else:
            (self.wTree.get_widget("deps_entry_licenses")).set_text("")
        
        writeListIntoTree(self.wTree.get_widget("deps_treeview_obsoletes"), package.obsoletes)
        writeListIntoTree(self.wTree.get_widget("deps_treeview_provides"), package.provides)
        writeListIntoTree(self.wTree.get_widget("deps_treeview_requires"), package.requires)
        
        writeIntoTextBuffer(self.wTree.get_widget("deps_textview_summary"), package.summary)
        
        href = self.wTree.get_widget("deps_href_url")
            
        if package.url!=None and package.url!="(none)":
            text = package.url
            if len(text)>27:
                text = text[:24]+"..."
            href.set_text(text)
            href.set_url(package.url)
            self.tooltips.set_tip(href, package.url)
        else:
            href.set_text('')
            href.set_url('')             
                            
        if package.vendor!=None and package.vendor!="(none)":
            (self.wTree.get_widget("deps_entry_vendor")).set_text(package.vendor)
        else:
            (self.wTree.get_widget("deps_entry_vendor")).set_text("")


    '''
    #
    #
    '''
    def pkgtree_selection_changed(self, selection, repoi):
        (model, iter) = selection.get_selected()
        if iter is not None:
            pkgid = model.get(iter, self._DEP_PKGID)[0]            
            self._set_package_info(repoi, pkgid)
        
