import os
import os.path
import sys
import time
from i18n import _

import yum

if yum.__version__<"3.0":
    import guyumlogger
else:
    import guyumlogger3 as guyumlogger
    
from gyprogbar import GuyumProgressBar
from gyfunctions import *

if yum.__version__>"2.4.9":
    from yum.misc import sortPkgObj
    
try:
    import readline
except:
    pass
    
import yum.Errors

class YumOutput:

    def failureReport(self, errobj):
        """failure output for failovers from urlgrabber"""
        
        self.errorlog(1, '%s: %s' % (errobj.url, str(errobj.exception)))
        self.errorlog(1, 'Trying other mirror.')
        raise errobj.exception


    def setupProgessCallbacks(self, progressbar, factor):
        """sets up the progress callbacks and various 
           output bars based on debug level"""

        # if we're below 2 on the debug level we don't need to be outputting
        # progress bars - this is hacky - I'm open to other options
        # One of these is a download
        if self.conf.debuglevel < 2 or not sys.stdout.isatty():
            self.repos.setProgressBar(None)
            self.repos.callback = None
        else:
            self.repos.setProgressBar(guyumlogger.GuyumProgressBar(progressbar, factor))
            self.repos.callback = CacheProgressCallback(self.log, self.errorlog,
                                                        self.filelog, progressbar, factor)
        # setup our failure report for failover
        freport = (self.failureReport,(),{})
        self.repos.setFailureCallback(freport)
        
        # setup our depsolve progress callback
        dscb = DepSolveProgressCallBack(self.log, self.errorlog)
        self.dsCallback = dscb
    
    
class DepSolveProgressCallBack:
    """provides text output callback functions for Dependency Solver callback"""
    
    def __init__(self, log, errorlog):
        """requires yum-cli log and errorlog functions as arguments"""
        self.log = log
        self.errorlog = errorlog
        self.loops = 0
    
    def pkgAdded(self, pkgtup, mode):
        modedict = { 'i': 'installed',
                     'u': 'updated',
                     'o': 'obsoleted',
                     'e': 'erased'}
        (n, a, e, v, r) = pkgtup
        modeterm = modedict[mode]
        self.log(2, '--->Package %s.%s %s:%s-%s set to be %s' % (n, a, e, v, r, modeterm))
        
    def start(self):
        self.loops += 1
        
    def tscheck(self):
        self.log(2, '-->Running transaction check')
        
    def restartLoop(self):
        self.loops += 1
        self.log(2, '-->Restarting Dependency Resolution with new changes.')
        self.log(3, '--->Loop Number: %d' % self.loops)
    
    def end(self):
        self.log(2, '-->Finished Dependency Resolution')

    
    def procReq(self, name, formatted_req):
        self.log(2, '-->Processing Dependency: %s for package: %s' % (formatted_req, name))
        
    
    def unresolved(self, msg):
        self.log(2, '-->Unresolved Dependency: %s' % msg)

    
    def procConflict(self, name, confname):
        self.log(2, '-->Processing Conflict: %s conflicts %s' % (name, confname))

    def transactionPopulation(self):
        self.log(2, '-->Populating transaction set with selected packages. Please wait.')
    
    def downloadHeader(self, name):
        self.log(2, '--->Downloading header for %s to pack into transaction set.' % name)
    
        
class CacheProgressCallback:

    '''
    The class handles text output callbacks during metadata cache updates.
    '''
    
    def __init__(self, log, errorlog, filelog=None, progbar=None, factor=0.0):
        self.log = log
        self.errorlog = errorlog
        self.filelog = filelog
        self.progbar = progbar
        self.factor = factor        
        self.nextchunk = 0
        

    def log(self, level, message):
        self.log(level, message)
        self.log.file_object.flush()

    def errorlog(self, level, message):
        if self.errorlog:
            self.errorlog(level, message)
            self.errorlog.file_object.flush()

    def filelog(self, level, message):
        if self.filelog:
            self.filelog(level, message)
            self.filelog.file_object.flush()


    def progressbar(self, current, total, name=None):
        chunksize = float(total)/10
        text = "Added %d / %d packages" % (current, total)
        if current>=self.nextchunk:
            updateprogressbar(self.progbar, 0.01, self.factor, text)
            self.nextchunk = self.nextchunk + chunksize
        else:
            gtk.gdk.threads_enter()
            self.progbar.set_text(text)
            gtk.gdk.threads_leave()

