
import sys
import traceback
import signal
import re
import fnmatch
import time
import os
import exceptions
import pkgtree


import yum
import callback
from rpmUtils.miscutils import formatRequire, compareEVR
import guyumpackage

if yum.__version__<"3.0":
    from yum.logger import Logger
    import guyumlogger
else:
    import logging
    import guyumlogger3 as guyumlogger

import rpm

from guyumpackage import computesize, uniquepackages, filterpackages
import yumif
from gyfunctions import updateprogressbar

version = "0.0.1"

oldrepokey = {'name':'name', 'bandwidth':'bandwidth', 'enablegroups':'enablegroups', 
            'gpgcheck':'gpgcheck', 'includepkgs':'includepkgs', 'keepalive':'keepalive',
            'proxy':'proxy', 'proxy_password':'proxy_password', 'proxy_username':'proxy_username',            
            'exclude': 'excludes', 'retries':'retries', 'throttle':'throttle', 'timeout':'timeout',
            'mirrorlist':'mirrorlistfn', 'cachedir':'cachedir', 'gpgkey':'gpgkey', 'pkgdir':'pkgdir',
            'hdrdir':'hdrdir'}



class GuyumQuery(yumif.GuyumYum):
    def __init__(self, pkgops = [], sackops = [], options = None, logcb=None):
        yumif.GuyumYum.__init__(self, logcb=logcb)
        self.searchlist= ['name']
        self.searchstring = ''        
        self.pkgtree = None

    
    #
    # Returns package for installation     
    def findpackage(self, packageid):
        for pkg in self.pkgs:
            if pkg.pkgId == packageid:
                return pkg    
        return None
     
     
    '''
    #
    '''
    def getnamedpackage(self, packageid):
        for pkg in self.packages:
           if pkg.packageid == packageid:
               return pkg
        return None                    


    def rpmdbsearch(self, packages):
        ''' runs local search in RPM database '''
        retpackages = []
        ts = rpm.TransactionSet()        
        
        for pkgtpl in packages:
            newpackages = ts.dbMatch('name', pkgtpl[0])
            name = pkgtpl[0]

            for newpackage in newpackages:
                cmp = compareEVR((name, pkgtpl[1], pkgtpl[4]), (name, newpackage['version'], newpackage['release']))
                if cmp==-1:
                    retpackages.append((pkgtpl[5], newpackage['version'], newpackage['release'], 'd', pkgtpl[0], pkgtpl[3]))
                if cmp==1:
                    retpackages.append((pkgtpl[5], newpackage['version'], newpackage['release'], 'u', pkgtpl[0], pkgtpl[3]))
                if cmp==0:
                    retpackages.append((pkgtpl[5], newpackage['version'], newpackage['release'], 'o', pkgtpl[0], pkgtpl[3]))
                
        return retpackages


    
    def searchpackage(self, package, searchlist, repoid, plugins, progressbar, factor):
        ''' Search for a package in a given repository '''
        if progressbar is not None:
            self.logcb(pbar1fract=0.01, pbar1factor=factor, pbar1text="Starting search in %s " % repoid)

        #self.doPluginSetup(None, yum.plugins.TYPE_CORE, '/usr/lib/yum-plugins', '/etc/yum/pluginconf.d')

        #for plugin in plugins:
        #    print "deactivate plugin "+plugin
        #    self.deactivatePlugin(plugin)        

        self.startYum(repoid, progressbar, factor)
        
        packagestosearch = []
        packagestosearch.append(package)
        self.searchstring = package
        packages = []        
        
        pkgs = self.searchPackages(searchlist, packagestosearch)
        
        if progressbar is not None:
            self.logcb(pbar1fract=0.125, pbar1factor=factor, pbar1text="Finish")
                
        localpackages = []
        remotepackages = []

        for pkg in pkgs:            
            appendpackage = 1
            newpkg = guyumpackage.guyumpackage()
            if pkg.repoid=='installed' or pkg.repoid==repoid:
                # divide: remote and locale packages            
                if isinstance(pkg, yum.sqlitesack.YumAvailablePackageSqlite) and pkg.repoid!='installed':
                    newpkg.remotepackage(pkg, 'i')
                    remotepackages.append(newpkg)
                    self.pkgs.append(pkg)                
                else:
                    newpkg.localpackage(pkg, 'o')
                    localpackages.append(newpkg)
                                        
        packages = uniquepackages(localpackages, remotepackages)            
        for localpkg in localpackages:
            packages.append(localpkg)
        for remotepkg in remotepackages:
            packages.append(remotepkg)

        self.packages = filterpackages(packages)

        return self.packages
        
    
    def maprepokeys(self, repository, key):
        retvalue = ""
        if yum.__version__>'2.4.9':
            retvalue = getattr(repository, key)
        else:
            retvalue = getattr(repository, oldrepokey[key])
        if key=='gpgkey':
            retvalue=''.join(retvalue)
        return retvalue
    
    
    def getrepovalue(self, repoid, key):
        repository = None
        retstring = None
        for repo in self.repos.repos.values():
            if repo.id==repoid:                
                repository = repo
        if repository:                        
            retstring = "%s" % self.maprepokeys(repository, key)
        return retstring


    def getrepos(self):
        retrepos = []
        for repo in self.repos.repos.values():            
            if repo.enabled==1:
                enabledstring = "1"
            else:
                enabledstring = "0"
            retrepos.append({'id':repo.id, 'name':repo.name, 'enabled':enabledstring});
        return retrepos


    def getplugins(self):
        retplugins = []
        if hasattr(self.plugins, "_plugins") and hasattr(self.plugins._plugins, "keys"):
            for plugin in self.plugins._plugins.keys():
                pluginconf = self.plugins._getpluginconf(plugin)
                #print "pluginconf: %s" % str(self.plugins._plugins[plugin][1].defaults())
                if yum.config.getOption(pluginconf, 'main', 'enabled', yum.config.BoolOption(False)):
                    enabledstring = "1"
                else:
                    enabledstring = "0"
                retplugins.append({'id':plugin, 'name':plugin, 'enabled':enabledstring});
        return retplugins

        
