import sys

try:
    import pygtk
    #tell pyGTK, if possible, that we want GTKv2
    pygtk.require("2.0")
except:
    #Some distributions come with GTK2, but not pyGTK
    pass

try:
    import gtk
    import gtk.glade
    import gobject
except:
    print "An error occured"
    sys.exit(1)

import gnome.ui
from misc import GuyumConst
import guyumquery
from gyfunctions import *

class RepoDialogTree:
    _REPOPROP_KEY = 0
    _REPOPROP_VALUE = 1
    
    properties = ["name", "bandwidth", "enablegroups", 
                "gpgcheck", "includepkgs", "keepalive", "proxy",
                "proxy_password", "proxy_username", "exclude",
                "retries", "throttle", "timeout", "mirrorlist",
                "cachedir", "gpgkey", "pkgdir", "hdrdir"]
    
    '''
    # constructor 
    ''' 
    def __init__(self):
        self.model = None
        self.treeview = None


    '''
    # add columns to treeview 
    ''' 
    def add_columns(self, treeview):
        if self.treeview == None:
            self.treeview = treeview
            
            # property column
            renderer = gtk.CellRendererText()
            column = gtk.TreeViewColumn( 'Property', renderer, text=self._REPOPROP_KEY )
            column.set_clickable(True)
            column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
            column.connect("clicked", treeviewSortIDSet, (treeview, self._REPOPROP_KEY))
            self.treeview.append_column(column)

            # property column
            column = gtk.TreeViewColumn( 'Value', renderer, text=self._REPOPROP_VALUE )
            column.set_clickable(True)
            column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
            self.treeview.append_column(column)

    '''
    #
    # add data to treemodel
    '''
    def add_data(self, repoq, repoid):
        if self.model!=None:
            self.model = None
        self.model = gtk.TreeStore( gobject.TYPE_STRING,
                        gobject.TYPE_STRING )

        iter = None
        for prop in self.properties:
            iter = self.model.append(None)
            repoprop = repoq.getrepovalue(repoid, prop)
            self.model.set(iter,
                           self._REPOPROP_KEY, prop,                           
                           self._REPOPROP_VALUE, repoprop)
        sortmodel = gtk.TreeModelSort(self.model)
        self.treeview.set_model(sortmodel)
        sortmodel.set_sort_column_id(self._REPOPROP_KEY, gtk.SORT_ASCENDING)        




'''
#
# repositories treemodel
'''
class RepoTree:
    _REPO_ID = 0
    _REPO_ENABLED  = 1
    
    '''
    # constructor 
    ''' 
    def __init__(self):
        self.model = None
        self.treeview = None
            
    '''
    # add columns to treeview 
    ''' 
    def add_columns(self, treeview):
        if self.treeview == None:
            self.treeview = treeview        
            # id column
            renderer = gtk.CellRendererText()
            column = gtk.TreeViewColumn( 'ID', renderer, text=self._REPO_ID )
            column.set_clickable(True)
            column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
            column.set_resizable(True)            
            self.treeview.append_column(column)
            column.connect("clicked", treeviewSortIDSet, (self.treeview, self._REPO_ID))
            
            # enabled column
            renderer = gtk.CellRendererToggle()
            renderer.connect('toggled', self.toggle)
            column = gtk.TreeViewColumn('Enabled', renderer, active=self._REPO_ENABLED )
            column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
            self.treeview.append_column(column)
    
    '''
    #
    # toggle "enabled"-toggle-button event
    '''
    def toggle(self, cell, path):
        iter = self.model.get_iter_from_string(path)
        self.model.set(iter, self._REPO_ENABLED,
                       not self.model.get_value(iter, self._REPO_ENABLED))

    '''
    #
    # add data to treemodel
    '''
    def add_data(self, data):
        if self.model!=None:
            self.model = None
            
        self.model = gtk.TreeStore( gobject.TYPE_STRING,
                                    gobject.TYPE_BOOLEAN )
        iter = None
        for item in data:
            iter = self.model.append(None)
            enabled=False
            if item['enabled']=='1':
                enabled=True
            self.model.set(iter,
                           self._REPO_ID, item['id'],                           
                           self._REPO_ENABLED, enabled)
        self.treeview.set_model(self.model)
        self.model.set_sort_column_id(self._REPO_ID, gtk.SORT_ASCENDING)        
        
    '''
    #
    # get enabled repositories as list
    '''
    def get_enabled_repos( self ):
        repos_enabled = []
        iter = self.model.iter_children( None )
        while iter:
            if self.model.get_value(iter, self._REPO_ENABLED):
                repos_enabled.append(self.model.get_value(iter, self._REPO_ID))
            iter = self.model.iter_next(iter)
        return repos_enabled

