

import sys
import traceback
import signal
import re
import fnmatch
import time
import os
import exceptions

from optparse import OptionParser

import yum
import yum.config
import yum.Errors
import yum.packages
from rpmUtils.arch import getArchList
from rpmUtils.miscutils import formatRequire, compareEVR
if yum.__version__<"3.0":
    from yum.logger import Logger
    import guyumlogger
else:
    import logging
    import guyumlogger3 as guyumlogger
    
if yum.__version__>"2.9.9":
    import output
    from output import CacheProgressCallback, DepSolveProgressCallBack
else:
    import guyumout
    from guyumout import CacheProgressCallback, DepSolveProgressCallBack

from urlgrabber.progress import TextMeter

import callback
import urlgrabber
import urlgrabber.grabber
import guyumpackage

import guyumout

import rpm

from guyumpackage import computesize, uniquepackages, filterpackages
from gyfunctions import updateprogressbar


class GuyumYum(yum.YumBase, guyumout.YumOutput):
    def __init__(self, pkgops = [], sackops = [], options = None, logcb=None):
        try:
            yum.YumBase.__init__(self)
        except Exception, e:
            traceback.print_exc(file=sys.stderr)
            raise e
        pkgops = ['queryformat']
        options = {'all': 0, 'file': 0, 'queryformat': None, 'archlist': None, 'grouppkgs': 'default', 'whatrequires': 0, 'querytags': 0, 'conffile': '/etc/yum.conf', 'query': 0, 'repoid': [], 'envra': 0, 'group': 0, 'show_dupes': 0, 'cache': 0, 'nevra': 0, 'source': 0, 'version': 0, 'obsoletes': 0, 'provides': 0, 'location': 0, 'nvr': 0, 'whatobsoletes': 0, 'whatprovides': 0, 'tempcache': 0, 'conflicts': 0, 'info': 0, 'resolve': 0, 'changelog': 0, 'list': 0, 'quiet': 0, 'groupmember': 0, 'alldeps': 0, 'pkgnarrow': 'repos', 'requires': 0, 'whatconflicts': 0}
        self.options = options
        self.pkgops = pkgops
        self.sackops = sackops
        self.pkgs = []   
        self.packages = []     
        self.depspkgs = []
        self.repolist = []
        self.totalamount = 0
        self.logfileobject = None  
        self.filelog = None
        # log callback for GUI
        
        self.logcb=None
        if logcb is not None:
            self.logcb = logcb
        try:
            self.errorlog=guyumlogger.GuyumLogger(threshold=3, \
                                                    file_object=sys.stderr, \
                                                    logcb = self.logcb)
            self.log = guyumlogger.GuyumLogger(threshold=3, \
                                                file_object=sys.stdout, \
                                                logcb = self.logcb)
            
            
        except Exception, e:
            traceback.print_exc(file=sys.stderr)
            raise e
            
        try:
            if yum.__version__ > '2.6.9' and yum.__version__ < '3.0':
                self.doStartupConfig()
            
            self.doConfigSetup()
        except Exception, e:
            traceback.print_exc(file=sys.stderr)
            raise e
            
            
    def startYum(self, repoid, progressbar, factor):
        ''' start yum process '''
        if progressbar is not None:
            self.logcb(pbar1fract=0.01, pbar1factor=factor, msg="Start", pbar1text="Start")
        # enable given repository, disable others
        for repo in self.repos.repos.values():            
            if repo.id==repoid:                                
                if repo.enabled==0:                    
                    try:
                        self.repos.enableRepo(repoid)                
                    except:
                        traceback.print_exc(file=sys.stderr)
            else: 
                try:                               
                    self.repos.disableRepo(repo.id)
                except:
                    traceback.print_exc(file=sys.stderr)
    
        # setup repository 
        try:
            if yum.__version__<="3.1.2":
                self.doRepoSetup(thisrepo=repoid)
            else:
                if yum.__version__=="3.1.3":
                    repos = yum.YumBase._getRepos(self, thisrepo=repoid)            
                else:
                    repos = yum.YumBase._getRepos(self, thisrepo=repoid, doSetup=True)            
                # Ugly hack
                self._pkgSack=None
        except Exception, e:            
            traceback.print_exc(file=sys.stderr)
            raise e
            
        else:            
            msg = "Preparing database..."
            if progressbar is not None:
                self.logcb(pbar1fract=0.125, pbar1factor=factor, msg=msg, pbar1text=msg)
                
            # setup rpm database
            try:
                if yum.__version__>="3.0":            
                    if yum.__version__<="3.1.2":
                        self.doRpmDBSetup()
                        self.doUpdateSetup()                        
                    else:
                        yum.YumBase._getUpdates(self)
                        yum.YumBase._getRpmDB(self)                
            except Exception, e:
                traceback.print_exc(file=sys.stderr)
                raise e                
            else:                   
                # setup progress callback  
                try:   
                    self.setupProgessCallbacks(progressbar, factor)
                except Exception, e:
                    traceback.print_exc(file=sys.stderr)
                    raise e                      
                else:          
                    # setup our depsolve progress callback
                    if yum.__version__<"3.0":
                        dscb = DepSolveProgressCallBack(self.log, self.errorlog)
                    else:
                        dscb = DepSolveProgressCallBack()
                    self.dsCallback = dscb                    
                    # setup package sacks            
                    try:        
                        if yum.__version__<="3.1.2":
                            self.doSackSetup(thisrepo=repoid)
                        else:
                            #self._pkgSack = None
                            yum.YumBase._getSacks(self, thisrepo=repoid)
                    except Exception, e:
                        traceback.print_exc(file=sys.stderr)
                        raise e                        
                    else:                    
                        # setup transaction
                        try:        
                            if yum.__version__<"3.0":
                                self.doTsSetup()
                        except Exception, e:
                            traceback.print_exc(file=sys.stderr)
                            raise e                            
                        else:
                            if progressbar is not None:
                                self.logcb(pbar1fract=0.75, pbar1factor=factor)
                                
        
    def deactivatePlugin(self, pluginname):
        if self.plugins is not None:
            if self.plugins._plugins[pluginname] is not None:
                self.plugins._plugins[pluginname] = yum.plugins.DummyYumPlugins()
            
